package eu.ocopomo.chat;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class ChatServlet extends HttpServlet {

    private Map<String, ChatSession> sessions = Collections.synchronizedMap(new HashMap<String, ChatSession>());
    private Map<String, ChatRoom> rooms = Collections.synchronizedMap(new HashMap<String, ChatRoom>());
    private Map<String, ChatUser> jids = Collections.synchronizedMap(new HashMap<String, ChatUser>());
    private int latency = 100;
    private int wait = 1000;
    private static final String SITE_ID = "ocopomo.eu";
    private static final String NULL_TIMESTAMP = "null_timestamp";
    private static final String NOT_OPENED_ROOM = "not_opened_room";
    
    @Override
    public void init() throws ServletException {
        super.init();
    }

    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        ChatSession session;
        JSONObject req, resp;
        
        try {
            req = new JSONObject(readJSON(request.getInputStream()));
            
            System.out.println("chat post request: " + req);

            if (req.has("sid")) {
                String sid = req.getString("sid");
                session = sessions.get(sid);
                session.setRid(req);
            } else {
                String sid = java.util.UUID.randomUUID().toString();
                //TEST CAST
                //if(req.getString("from").equals("butka@ocopomo.eu")) sid = "butkasid";
                //if(req.getString("from").equals("bednar@ocopomo.eu")) sid = "bednarsid";
                //END
                String from = req.getString("from");
                if(!jids.containsKey(from)){
                    createChatUser(from);
                }
                session = getNewSession(sid, req.getString("from"), req.getInt("rid"));
                serialize(createSessionResponse(req, sid), response);
                return;
            }

            if (req.getString("body").equals("[]") == false) {
                JSONArray body = req.getJSONArray("body");
                for (int i = 0; i < body.length(); i++) {
                    JSONObject obj = body.getJSONObject(i);
                    String type = obj.getString("type");
                    if(!jids.containsKey(obj.getString("from"))){
                        createChatUser(obj.getString("from"));
                    }
                    if (type.equals("presence")) {
                        if(isRoomId(obj.getString("to"))){
                            ChatRoom room = rooms.get(obj.getString("to"));
                            String from = obj.getString("from");
                            ChatUser chuser = jids.get(from);
                            if(room.hasOccupant(from)){
                                if(obj.has("close")){
                                    closeRoomForJid(room.getRoomId(), chuser);
                                }else{
                                    DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SS'Z'");
                                    String stamp = df.format(new Date());
                                    chuser.setTimestamp(room.getRoomId(), stamp);
                                    sendRoomHistoryToSpecificSession(room, req.getString("sid"));
                                }
                            }
                            else{
                                chuser.setTimestamp(room.getRoomId(), NULL_TIMESTAMP);
                                addNewUserToChat(from, room);
                            }
                        }
                        else{
                            sendSitePresence(obj);
                        }
                    }
                    if (type.equals("message")) {
                        String to = obj.getString("to");
                        DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SS'Z'");
                        String stamp = df.format(new Date());
                        obj.put("stamp",stamp);
                        if (isRoomId(to)) {
                            ChatRoom room = rooms.get(to);
                            for(String h : room.getJids()){
                                ChatUser chuser = jids.get(h);
                                if(!chuser.getOpenedRoomFlag().equals(NOT_OPENED_ROOM)){
                                    chuser.setTimestamp(chuser.getOpenedRoomFlag(), stamp);
                                }
                            }
                            sendMessageToRoom(obj);
                        } else {
                            if(!jids.containsKey(to)){
                                createChatUser(to);
                            }
                            ChatRoom room = createChatRoom(obj);
                            String from = obj.getString("from");
                            room.addOccupantToRoom(from);
                            ChatUser creator = jids.get(from);
                            room.addOccupantToRoom(to);
                            ChatUser other = jids.get(to);
                            creator.addToRooms(room.getRoomId());
                            other.addToRooms(room.getRoomId());
                            sendPresence(from, room.getRoomId(), from);
                            sendPresence(to ,room.getRoomId(), to);
                            sendPresence(from, room.getRoomId(), to);
                            sendPresence(to ,room.getRoomId(), from);
                            obj.remove("to");
                            obj.put("to", room.getRoomId());
                            creator.setTimestamp(room.getRoomId(), stamp); 
                            other.setTimestamp(room.getRoomId(), NULL_TIMESTAMP);
                            sendMessageToRoom(obj);
                        }
                    }
                    if(type.equals("iq")){
                        if(isRoomId(obj.getString("to"))){
                            createHistoryQueryResult(obj.getString("to"), req.getString("sid"), req.getInt("rid"), response);
                        }else{
                            String from = obj.getString("from");
                            Set<String> myrooms = jids.get(from).getRooms();
                            if(req.has("flush")) sessions.get(req.getString("sid")).flushQueue();
                            createQueryRoomsResult(myrooms, jids.get(from).getTimestamps(), req.getString("sid"), req.getInt("rid"), response);
                        }
                        return;
                    }
                }
            }else{
                if(req.has("openedRoom")){
                    jids.get(session.getJid()).setOpenedRoomFlag(req.getString("openedRoom"));
                }else{
                    jids.get(session.getJid()).setOpenedRoomFlag(NOT_OPENED_ROOM);
                }
            }

            int t = 0;
            while (t < wait) {
                resp = session.createResponse(req);
                if (resp != null) {
                    serialize(resp, response);
                    return;
                }
                try {
                    Thread.sleep(latency);
                } catch (InterruptedException e) {
                    throw new IOException(e);
                }
                t += latency;
            }

            serialize(createEmptyResponse(req), response);

        } catch (JSONException ex) {
            throw new IOException(ex);
        } 

    }

    private void serialize(JSONObject obj, HttpServletResponse resp) throws IOException {
        System.out.println("chat post response: " + obj);
        resp.getWriter().print(obj.toString());
    }

    private String readJSON(InputStream is) throws UnsupportedEncodingException, IOException {
        final char[] buffer = new char[0x10000];
        StringBuilder out = new StringBuilder();
        Reader in = new InputStreamReader(is, "UTF-8");
        int read;
        do {
            read = in.read(buffer, 0, buffer.length);
            if (read > 0) {
                out.append(buffer, 0, read);
            }
        } while (read >= 0);
        return out.toString();
    }

    private JSONObject createEmptyResponse(JSONObject req) throws JSONException {
        JSONObject resp = new JSONObject();
        resp.put("sid", req.get("sid"));
        resp.put("rid", req.getInt("rid"));
        resp.put("body", "[]");
        return resp;
    }

    private JSONObject createSessionResponse(JSONObject req, String id) throws JSONException {
        JSONObject resp = new JSONObject();
        resp.put("sid", id);
        resp.put("rid", req.getInt("rid"));
        return resp;
    }

    private ChatSession getNewSession(String sid, String jid, int rid) {
        if (jids.get(jid).hasSessions()) {
            for (String h : jids.get(jid).getSessions()) {
                ChatSession st = sessions.get(h);
                if (st.getRid() == ChatSession.NOT_USED_RID) {
                    st.setSid(sid);
                    st.setRid(rid);
                    jids.get(jid).removeFromSessions(h);
                    jids.get(jid).addToSessions(sid);
                    sessions.remove(h);
                    sessions.put(sid, st);
                    return st;
                }
            }
        }

        ChatSession session = new ChatSession(sid, jid, rid);
        sessions.put(sid, session);
        jids.get(jid).addToSessions(sid);
        return session;
    }

    private void sendMessageToRoom(JSONObject msg) throws JSONException {
        String roomId = msg.getString("to");
        rooms.get(roomId).addToHistory(msg);
        if(msg.has("subject")) rooms.get(roomId).setSubject(msg.getString("subject"));
        Set<String> ids = rooms.get(roomId).getJids();
        for (String id : ids) {
            Set<String> jsStr = jids.get(id).getSessions();
            if (jsStr.isEmpty()) {
                ChatSession newses = createNewEmptySession(java.util.UUID.randomUUID().toString(), id);
                newses.addToQueue(msg);
                return;
            }
            for (String s : jsStr) {
                ChatSession ses = sessions.get(s);
                ses.addToQueue(msg);
            }
        }
    }

    private ChatSession createNewEmptySession(String sid, String jid) {
        ChatSession session = new ChatSession(sid, jid, ChatSession.NOT_USED_RID);
        sessions.put(sid, session);
        jids.get(jid).addToSessions(sid);
        return session;
    }
    
    private ChatRoom createChatRoom(JSONObject msg) throws JSONException{
        String roomId = "room://" + java.util.UUID.randomUUID().toString() + "@" + SITE_ID;
        //TEST CAST
        //if(rooms.isEmpty()) roomId = "room://1@ocopomo.eu";
        //END
        ChatRoom room = new ChatRoom(roomId, new HashSet<String>(), msg.getString("body"), new ArrayList<JSONObject>());
        rooms.put(roomId, room);
        return room;
    }

    private void sendPresence(String jid, String roomId, String from) throws JSONException {
        JSONObject msg = new JSONObject();
        msg.put("from", from);
        msg.put("to", roomId);
        msg.put("type", "presence");
        msg.put("subject", rooms.get(roomId).getSubject());
        sendMsgToSessions(jid, msg);
    }
    
    private void sendSitePresence(JSONObject obj) throws JSONException{
        for(String jid : getUserWithJidInRoster(obj)){
            sendMsgToSessions(jid, obj);
        }
    }
    
    private Set<String> getUserWithJidInRoster(JSONObject obj) throws JSONException{
        Set<String> list = new HashSet<String>();
        for(ChatUser c : jids.values()){
            if(c.hasInRoster(obj.getString("from"))) list.add(c.getJid());
        }
        return list;
    }
    
    private void addNewUserToChat(String from, ChatRoom room) throws JSONException{
        room.addOccupantToRoom(from);
        jids.get(from).addToRooms(room.getRoomId());
        Set<String> occupants = room.getJids();
        for(String oc : occupants){
            sendPresence(oc, room.getRoomId(), from);
            if(!oc.equals(from)) sendPresence(from, room.getRoomId(), oc);
        }
        sendRoomHistory(from,room);
    }
    
    private void sendMsgToSessions(String jid, JSONObject msg){
        Set<String> jsStr = jids.get(jid).getSessions();
        if (jsStr.isEmpty()) {
            ChatSession newses = createNewEmptySession(java.util.UUID.randomUUID().toString(), jid);
            newses.addToQueue(msg);
            return;
        }
        for (String s : jsStr) {
            ChatSession ses = sessions.get(s);
            ses.addToQueue(msg);
        }
    }
    
    private void sendRoomHistory(String jid, ChatRoom room) throws JSONException{
        List<JSONObject> history = room.getHistory();
        Set<String> jsStr = jids.get(jid).getSessions();
        if (jsStr.isEmpty()) {
            ChatSession newses = createNewEmptySession(java.util.UUID.randomUUID().toString(), jid);
            for(JSONObject o : history){
                newses.addToQueue(o);
            }
            return;
        }
        for (String s : jsStr) {
            ChatSession ses = sessions.get(s);
            for(JSONObject o : history){
                o.put("history", "true");
                ses.addToQueue(o);
            }
        }
    }
    
    private void sendRoomHistoryToSpecificSession(ChatRoom room, String sid) throws JSONException{
        List<JSONObject> history = room.getHistory();
        ChatSession ses = sessions.get(sid);
        for(JSONObject o : history){
            o.put("history", "true");
            ses.addToQueue(o);
        }
    }
    
    private boolean isRoomId(String id) {
        if (rooms.containsKey(id)){
            return true;
        } else {
            return false;
        }
    }
    
    private void closeRoomForJid(String roomId, ChatUser chuser) throws JSONException {
        ChatRoom room = rooms.get(roomId);
        room.removeOccupantFromRoom(chuser.getJid());
        chuser.removeFromRooms(roomId);
        chuser.removeTimestamp(roomId);
        if(room.getJids().isEmpty()){
            rooms.remove(roomId);
        }else{
            Set<String> occupants = room.getJids();
            for(String oc : occupants){
                JSONObject msg = new JSONObject();
                msg.put("from", chuser.getJid());
                msg.put("to", roomId);
                msg.put("type", "presence");
                msg.put("close", "true");
                sendMsgToSessions(oc, msg);
            }
        }
    }
    
    private void createHistoryQueryResult(String roomid, String sid, int rid, HttpServletResponse res) throws JSONException, IOException {
        JSONObject obj = new JSONObject();
        obj.put("sid",sid);
        obj.put("rid",rid);
        JSONObject obja = new JSONObject();
        obja.put("type", "result");
        obja.put("history", rooms.get(roomid).getHistory());
        System.out.println("history = " + obja.get("history"));
        JSONArray arr = new JSONArray();
        arr.put(obja);
        obj.put("body", arr);
        serialize(obj, res);
    }
    
    private void createQueryRoomsResult(Set<String> myrooms, Map<String,String> timestamps, String sid, int rid, HttpServletResponse res) throws JSONException, IOException {
        JSONObject obj = new JSONObject();
        obj.put("sid",sid);
        obj.put("rid",rid);
        JSONObject obja = new JSONObject();
        obja.put("type", "result");
        JSONArray items = new JSONArray();
        for(Iterator<String> i = myrooms.iterator(); i.hasNext();){
            ChatRoom r = rooms.get(i.next());
            JSONObject robj = new JSONObject();
            robj.put("roomid", r.getRoomId());
            robj.put("subject", r.getSubject());
            robj.put("occupants", r.getJids());
            List<JSONObject> his = r.getHistory();
            String ts = timestamps.get(r.getRoomId());
            int newm = 0;
            if(ts.equals(NULL_TIMESTAMP)){
                newm = his.size();
            }else{
                for(int j=0;j<his.size();j++){
                    boolean found = false;
                    try {
                        found = isNewerMessage(his.get(j),ts);
                    } catch (ParseException ex) {
                        Logger.getLogger(ChatServlet.class.getName()).log(Level.SEVERE, null, ex);
                    }
                    if(found == true) {
                        newm = his.size() - j - 1;
                        break;
                    }  
                }
            }
            robj.put("newmsg", newm);
            items.put(robj);
        }
        obja.put("items", items);
        JSONArray arr = new JSONArray();
        arr.put(obja);
        obj.put("body", arr);
        serialize(obj, res);
    }

    private boolean isNewerMessage(JSONObject o, String ts) throws JSONException, ParseException {
        String ots = o.getString("stamp");
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SS'Z'");
        if(df.parse(ots).after(df.parse(ts))) return true;
        else return false;
    }

    private void createChatUser(String jid) {
        ChatUser u = new ChatUser(jid, new HashSet<String>());
        Iterator<String> iterator = jids.keySet().iterator();
        while (iterator.hasNext()) {
           String i = iterator.next();
           u.addToRoster(jids.get(i).getJid());
           jids.get(i).addToRoster(jid);
        }
        jids.put(jid, u);
    }

}
