package eu.ocopomo.chat;

import java.util.LinkedList;
import java.util.Queue;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class ChatSession {

    private String sid;
    private String jid;
    private int rid;
    private Queue<JSONObject> queue;
    
    public static final int NOT_USED_RID = -1;
    
    public ChatSession(String sid, String jid, int rid) {
        this.sid = sid;
        this.jid = jid;
        this.rid = rid;
        this.queue = new LinkedList<JSONObject>();
    }
    
    public synchronized void addToQueue(JSONObject msg) {
        this.queue.add(msg);
    }
    
    public synchronized void flushQueue(){
        this.queue = new LinkedList<JSONObject>();
    }    
    
    public synchronized JSONObject createResponse(JSONObject req) throws JSONException {
        JSONObject resp = new JSONObject();
        
        resp.put("sid", this.sid);
        resp.put("rid", req.getInt("rid"));

        if (req.getInt("rid") != this.rid) {
            resp.put("body", "[]");
        } else {
            if (queue.isEmpty()) {
                return null;
            }
            resp.put("body", new JSONArray(queue));
            queue.clear();
        }

        return resp;
    }

    public synchronized boolean setRid(JSONObject req) throws JSONException {
        if(this.rid < req.getInt("rid")){
            this.rid = req.getInt("rid");
            return true;
        }
        return false;
    }
    
    public String getJid() {
        return jid;
    }

    public void setJid(String jid) {
        this.jid = jid;
    }

    public int getRid() {
        return rid;
    }

    public void setRid(int rid){
        this.rid = rid;
    }
    
    public String getSid() {
        return sid;
    }

    public void setSid(String sid) {
        this.sid = sid;
    }
    
}
