/**
 * Dashboard Chat component.
 * 
 * @namespace Alfresco.dashlet
 * @class Alfresco.dashlet.Chat
 */
(function()
{
    /**
    * YUI Library aliases
    */
    var Event = YAHOO.util.Event;
    var Dom = YAHOO.util.Dom;
	var Connect = YAHOO.util.Connect;
	var Element = YAHOO.util.Element;
    
	var CV_CON = "contacts";
	var CV_CHS = "chats";
	var CV_CHA = "chat";
	
	/**
    * Dashboard Chat constructor.
    * 
    * @param {String} htmlId The HTML id of the parent element
    * @return {Alfresco.dashlet.Chat} The new component instance
    * @constructor
    */
    Alfresco.dashlet.Chat = function (htmlId)
    {
        return Alfresco.dashlet.Chat.superclass.constructor.call(this, "Alfresco.dashlet.Chat", htmlId);
    };

    YAHOO.extend(Alfresco.dashlet.Chat, Alfresco.component.Base,
    {
        /**
       * Object container for initialization options
       *
       * @property options
       * @type {object} object literal
       */
        options:
        {
            /**
          * @property siteId
          * @type String
          * */
            siteId : "",
        },
		
		chatsiteorg : "ocopomo.eu",
		chatsid : null,
		chatrid : 0,
		sUrl : "http://ocopomo.ekf.tuke.sk/chat/ChatServlet", // setup correct URL for Chat Serlvet here !!!
		jid : null,
		otherid : null,
		roomid: null,
		roster: [],
		rooms: [],
		currentview: CV_CON,
		startingSubject: "",
		openedRoom: "NOR",
		qnum: 0,
		searchATC: [],
		temp_history: null,
		temp_filter: "",
		temp_dialog: null,
		users: [],
		
		/**
       * Fired by YUI when parent element is available for scripting
       * @method onReady
       */
        onReady: function ()
        {
			var me = this;
			
			this.getUsers();
			
			YAHOO.namespace("example.container");
			
			this.jid =  Alfresco.constants.USERNAME + "@" + this.chatsiteorg;
			
			this.widgets.range = new YAHOO.widget.Button(this.id + "-contactsrange",
			{
				type: "split",
				menu: this.id + "-contactsrange-menu",
				lazyloadmenu: false
			});
			
			this.widgets.range.on("click", this.onContactsFilterClicked, this, true);
			this.widgets.range.getMenu().subscribe("click", function (p_sType, p_aArgs)
			{
				var menuItem = p_aArgs[1];
				if (menuItem)
				{
					me.widgets.range.set("label", menuItem.cfg.getProperty("text"));
					me.onContactsFilterChanged.call(me, p_aArgs[1]);
				}
			});
			
			this.widgets.range2 = new YAHOO.widget.Button(this.id + "-chatsrange",
			{
				type: "split",
				menu: this.id + "-chatsrange-menu",
				lazyloadmenu: false
			});
			
			this.widgets.range2.on("click", this.onChatsFilterClicked, this, true);
			this.widgets.range2.getMenu().subscribe("click", function (p_sType, p_aArgs)
			{
				var menuItem = p_aArgs[1];
				if (menuItem)
				{
					me.widgets.range2.set("label", menuItem.cfg.getProperty("text"));
					me.onChatsFilterChanged.call(me, p_aArgs[1]);
				}
			});
			
			var handleSubmitChangeSub = function() {
				var val = this.validate();
				if(val == true) this.cancel();
			};
			var handleCancelChangeSub = function() {
				this.cancel();
			};
			
			YAHOO.example.container.dialogChangeSub = new YAHOO.widget.Dialog("dialogChangeSub",  
	            { width : "400px", 
	              fixedcenter : true, 
	              visible : false,
				  modal: true,
				  hideaftersubmit: true,
	              constraintoviewport : true,
				  buttons : [ { text:"Submit", handler:handleSubmitChangeSub, isDefault:true }, 
	                          { text:"Cancel", handler:handleCancelChangeSub } ] 
	             } ); 
			
			YAHOO.example.container.dialogChangeSub.validate = function() { 
				var data = this.getData(); 
				var id = me.getRoomsObjectId(me.roomid, me.rooms);
				if (me.isBlank(data.subject)) { 
					alert(me.msg("alert.changesubject.empty") + " !"); 
					return false; 
				} 
				if (data.subject == me.rooms[id].subject) {
					alert(me.msg("alert.changesubject.issame") + " !");
					return false;
				}
				me.sendNewSubject(data.subject, me.roomid, me);
				return true; 
			}; 
			
			YAHOO.example.container.dialogChangeSub.changeSubShow = function(){
				var id = me.getRoomsObjectId(me.roomid, me.rooms);
				this.form.subject.value = me.rooms[id].subject;
				this.show();
			};
			
			YAHOO.example.container.dialogChangeSub.render(document.body);
			YAHOO.util.Event.addListener(this.id + "-changesubject-button", "click", YAHOO.example.container.dialogChangeSub.changeSubShow, YAHOO.example.container.dialogChangeSub, true);
			
			var handleSubmitAddToChat = function() {
				var val = this.validate();
				if(val == true) {
					me.searchATC = [];
					this.cancel();
				}
			};
			var handleCancelAddToChat = function() {
				me.searchATC = [];
				this.cancel();
			};
			
			YAHOO.example.container.dialogAddToChat = new YAHOO.widget.Dialog("dialogAddToChat",  
	            { width : "450px", 
	              fixedcenter : true, 
	              visible : false,
				  modal: true,
				  hideaftersubmit: true,
	              constraintoviewport : true,
				  buttons : [ { text:"Add", handler:handleSubmitAddToChat, isDefault:true }, 
	                          { text:"Cancel", handler:handleCancelAddToChat }] 
	             } ); 
			
			YAHOO.example.container.dialogAddToChat.validate = function() { 
				var list = Dom.get(me.id + '-addtochat-list');
				if(me.searchATC.length == 0){
					alert(me.msg("alert.addtochat.empty") + " !"); 
					return false;
				}
				var selected = [];
				for(var i=0;i<me.searchATC.length;i++){
					var input = Dom.get(me.id + '-addtochat-checkbox-' + me.searchATC[i].userName);
					if(input.checked == true) selected.push(me.searchATC[i].userName + "@" + me.chatsiteorg);
				}
				if(selected.length == 0){
					alert(me.msg("alert.addtochat.noselecteduser") + " !");
					return false;
				}else{
					var id = me.getRoomsObjectId(me.roomid, me.rooms);
					me.addSelectedToChat(selected, id);
				}
				return true; 
			}; 
			
			YAHOO.example.container.dialogAddToChat.addToChatShow = function(){
				var con = Dom.get(me.id + '-addtochat-container');
				con.innerHTML = "";
				var inp = Dom.get(me.id + '-dialogAddToChat-search-input');
				inp.value = "";
				me.searchATC = [];
				this.show();
			};
			
			YAHOO.example.container.dialogAddToChat.render(document.body);
			YAHOO.util.Event.addListener(this.id + "-addtochat-button", "click", YAHOO.example.container.dialogAddToChat.addToChatShow, YAHOO.example.container.dialogAddToChat, true);
			Event.on(this.id + "-dialogAddToChat-search-button", "click", this.onAddToChatDialogSearchClick, null, this);
			
			var handleSubmitPublish = function() {
				var val = this.validate();
				if(val == true) this.cancel();
			};
			var handleCancelPublish = function() {
				me.temp_dialog = null;
				this.cancel();
			};
			
			YAHOO.example.container.dialogPublish = new YAHOO.widget.Dialog("dialogPublish",  
	            { width : "400px", 
	              fixedcenter : true, 
	              visible : false,
				  modal: true,
				  hideaftersubmit: true,
	              constraintoviewport : true,
				  buttons : [ { text:"Submit", handler:handleSubmitPublish, isDefault:true }, 
	                          { text:"Cancel", handler:handleCancelPublish } ] 
	             } ); 
			
			YAHOO.example.container.dialogPublish.validate = function() { 
				var data = this.getData(); 
				var id;
				if(me.roomid !== null){
					id = me.getRoomsObjectId(me.roomid, me.rooms);
				}else{
					id = me.getRoomsObjectId(temp_dialog, me.rooms);
				}
				if (me.isBlank(data.name)) { 
					alert(me.msg("alert.publish.emptyname") + " !"); 
					return false; 
				}
				var occ = [];
				for(var i=0;i<me.rooms[id].occupants.length;i++){
					occ.push(me.getUsernameFromJid(me.rooms[id].occupants[i]));
				}
				var roomhistory = {
					name: data.name,
					title: data.title,
					description: data.description,
					subject: me.rooms[id].subject,
					occupants: occ,
					history: []
				};
				me.createDocumentFromHistory(me.rooms[id].roomid, me, roomhistory);
				temp_dialog = null;
				return true; 
			}; 
			
			YAHOO.example.container.dialogPublish.publishShow = function(e){
				var id = "";
				if(me.roomid !== null){ 
					id = me.getRoomsObjectId(me.roomid, me.rooms);
				}else{
					id = me.getRoomsObjectId(me.getRoomidFromPublishLink(e.currentTarget.id), me.rooms);
					temp_dialog = me.getRoomidFromPublishLink(e.currentTarget.id);
				}
				this.form.name.value = me.rooms[id].subject;
				this.form.title.value = "";
				this.form.description.value = "";
				this.show();
			};
			
			YAHOO.example.container.dialogPublish.render(document.body);
			YAHOO.util.Event.addListener(this.id + "-publish-button", "click", YAHOO.example.container.dialogPublish.publishShow, YAHOO.example.container.dialogPublish, true);
			
			var handleSubmitCreateDiscussion = function() {
				var val = this.validate();
				if(val == true) this.cancel();
			};
			var handleCancelCreateDiscussion = function() {
				this.cancel();
			};
			
			YAHOO.example.container.dialogCreateDiscussion = new YAHOO.widget.Dialog("dialogCreateDiscussion",  
	            { width : "400px", 
	              fixedcenter : true, 
	              visible : false,
				  modal: true,
				  hideaftersubmit: true,
	              constraintoviewport : true,
				  buttons : [ { text:"Submit", handler:handleSubmitCreateDiscussion, isDefault:true }, 
	                          { text:"Cancel", handler:handleCancelCreateDiscussion } ] 
	             } ); 
			
			YAHOO.example.container.dialogCreateDiscussion.validate = function() { 
				var data = this.getData(); 
				if (me.isBlank(data.name)) { 
					alert(me.msg("alert.creatediscussion.emptyname") + " !"); 
					return false; 
				}
				if(me.isBlank(data.text)){
					alert(me.msg("alert.creatediscussion.emptytext") + " !"); 
					return false; 
				}
				var obj = {
					title: data.name,
					content: data.text,
					tags: ["chat"]
				};
				me.createDiscussionTopic(me, obj);
				return true; 
			}; 
			
			YAHOO.example.container.dialogCreateDiscussion.createDiscussionShow = function(e){
				var id = "";
				if(me.roomid !== null){ 
					id = me.getRoomsObjectId(me.roomid, me.rooms);
				}else{
					id = me.getRoomsObjectId(me.getRoomidFromCreateDiscussionLink(e.currentTarget.id), me.rooms);
				}
				this.form.name.value = me.rooms[id].subject;
				this.form.text.value = "";
				this.show();
			};
			
			YAHOO.example.container.dialogCreateDiscussion.render(document.body);
			YAHOO.util.Event.addListener(this.id + "-creatediscussion-button", "click", YAHOO.example.container.dialogCreateDiscussion.createDiscussionShow, YAHOO.example.container.dialogCreateDiscussion, true);
			
			this.getRoster(this.widgets.range._button.textContent);
			this.makeConnection();
        },
		
		getUsers: function(){
			Alfresco.util.Ajax.request(
			{
				url: Alfresco.constants.PROXY_URI + "api/people",
				successCallback:
				{
					fn: this.setupUsers,
					scope: this
				}
			});
		},
		
		setupUsers: function(resp){
			var ps = [];
			ps = resp.json.people;
			this.users = [];
			for (var i=0; i< ps.length; i++)
			{
				var p = ps[i];
				if(p.userName !== "guest"){
					var user = {
						username: p.userName,
						fullname: p.firstName + ' ' + p.lastName
					};
					this.users.push(user);
				}
			}
		},
		
		onAddToChatDialogSearchClick: function(){
			var container = Dom.get(this.id + '-addtochat-container');
			container.innerHTML = "";
			this.searchATC = [];
			var input = Dom.get(this.id + '-dialogAddToChat-search-input');
			this.getPeopleSearchATC(input.value);
		},
		
		onContactsFilterClicked: function (e){
			if(this.currentview !== CV_CON){
				this.currentview = CV_CON;
				this.getRoster(this.widgets.range._button.textContent);
			}
			this.roomid = null;
			this.otherid = null;
			this.openedRoom = "NOR";
			Dom.get(this.id + '-title').innerHTML = "";
		},
		
		onContactsFilterChanged: function (p_oMenuItem)
		{
			this.widgets.range.value = p_oMenuItem.value;
			if(this.currentview !== CV_CON){
				this.currentview = CV_CON;
			}
			this.roomid = null;
			this.otherid = null;
			this.openedRoom = "NOR";
			this.getRoster(this.widgets.range._button.textContent);
			Dom.get(this.id + '-title').innerHTML = "";
		},
		
		onChatsFilterClicked: function (e){
			if(this.currentview !== CV_CHS){
				this.currentview = CV_CHS;
				this.showChatsList(this.widgets.range2._button.textContent);
			}
			this.roomid = null;
			this.otherid = null;
			this.openedRoom = "NOR";
			Dom.get(this.id + '-title').innerHTML = "";
		},
		
		onChatsFilterChanged: function (p_oMenuItem)
		{
			this.widgets.range.value = p_oMenuItem.value;
			if(this.currentview !== CV_CHS){
				this.currentview = CV_CHS;
			}
			this.roomid = null;
			this.otherid = null;
			this.openedRoom = "NOR";
			this.showChatsList(this.widgets.range2._button.textContent);
			Dom.get(this.id + '-title').innerHTML = "";
		},
		
		addSelectedToChat: function (selected, id){
			for(var i=0;i<selected.length;i++){
				var postDataATC = '{"rid": ' + this.chatrid + ',"sid":"' + this.chatsid + '","body":[{"type":"presence","from":"' + selected[i] + '", "to":"' + this.rooms[id].roomid + '"}]}'; 
				Connect.asyncRequest('POST', this.sUrl, {success: this.handleSuccess, failure: this.handleFailure, argument: this}, postDataATC);
				this.chatrid++;
			}
		},
		
		showChatsList: function(filter)
		{
			var divtoolbar = Dom.get(this.id + '-toolbarlinks');
			divtoolbar.innerHTML = "";
			var con = Dom.get(this.id + '-container');
			con.innerHTML = "";
			var myrooms = [];
			if(filter == this.msg("filter.chats.recent")){
				for(var h=0;h<this.rooms.length;h++){
					if(this.rooms[h].newmsg > 0) myrooms.push(this.rooms[h]);
				}
			}
			if(filter == this.msg("filter.chats.all")){
				myrooms = this.rooms;
			}
			if(myrooms.length > 0){
				var divchats = document.createElement('div');
				divchats.setAttribute('id',this.id + '-chatslist');
				divchats.setAttribute('class','body scrollableList');
				con.appendChild(divchats);
				for (var i=0; i<myrooms.length; i++){
					var ch = myrooms[i];
					var newdiv = document.createElement('div');
					if(i == 0){
						if(myrooms.length == 1) newdiv.setAttribute('class', 'detail-list-item first-item last-item');
						else newdiv.setAttribute('class', 'detail-list-item first-item');
					}
					if(i == myrooms.length - 1 && i > 0) newdiv.setAttribute('class', 'detail-list-item last-item');
					if(i !== myrooms.length - 1 && i > 0) newdiv.setAttribute('class', 'detail-list-item');
					divchats.appendChild(newdiv);
					
					var divtable = document.createElement('table');
					var divtr = document.createElement('tr');
					var divtd1 = document.createElement('td');
					var divtd2 = document.createElement('td');
					divtd2.setAttribute('style','border-left: 1px solid black; padding-left: 10px;');
					newdiv.appendChild(divtable);
					divtable.appendChild(divtr);
					divtr.appendChild(divtd1);
					divtr.appendChild(divtd2);
					
					var divperson = document.createElement('div');
					var str = '<h4><a id="' + this.id + '-openchatlinkFL-' + ch.roomid + '" href="javascript: void(0)" class="theme-color-1 openchatlink">' + ch.subject + '</a></h4>';
					divperson.innerHTML += str;
					str = '<div>' + this.msg("chatlist.with") + ':</div>';
					divperson.innerHTML += str;
					divperson.innerHTML += '<div id="' + this.id + '-chatlistwith-' + ch.roomid + '">';
					for(var j=0;j<ch.occupants.length;j++){
						var occ = ch.occupants[j];
						if(this.jid !== occ){
							divperson.innerHTML += '<li>' + this.getFullnameFromJid(occ); + '</li>';
						}
					}
					divperson.innerHTML += '</div>';
					divtd1.appendChild(divperson);
					
					var divrp = document.createElement('div');
					divrp.setAttribute('id',this.id + '-newmsg-' + ch.roomid);
					divrp.innerHTML = this.msg("chat.new") + ': ';
					if(ch.newmsg > 0){ divrp.innerHTML += '<span style="color: red;">' + ch.newmsg + '</span>';}
					else {divrp.innerHTML += ch.newmsg;}
					var divlinks = document.createElement('div');
					divlinks.innerHTML = '<a id="' + this.id + '-publish-button-' + ch.roomid + '" href="javascript: void(0)" class="theme-color-1">' + this.msg("link.publish") + '</a>';
					divlinks.innerHTML += '<br/>';
					divlinks.innerHTML += '<a id="' + this.id + '-creatediscussion-button-' + ch.roomid + '" href="javascript: void(0)" class="theme-color-1">' + this.msg("link.creatediscussion") + '</a>';
					divlinks.innerHTML += '<br/>';
					divlinks.innerHTML += '<a id="' + this.id + '-closechatlinkFL-' + ch.roomid + '" href="javascript: void(0)" class="theme-color-1">' + this.msg("link.close") + '</a>';
					divtd2.setAttribute('width','15%');
					divtd2.appendChild(divrp);
					divtd2.appendChild(divlinks);
					
					Event.on(this.id + "-openchatlinkFL-"  + ch.roomid, "click", this.onExistingChatLinkClick, null, this);
					Event.on(this.id + "-closechatlinkFL-"  + ch.roomid, "click", this.onCloseChatLinkClick, null, this);
					YAHOO.util.Event.addListener(this.id + "-publish-button-" + ch.roomid, "click", YAHOO.example.container.dialogPublish.publishShow, YAHOO.example.container.dialogPublish, true);
					YAHOO.util.Event.addListener(this.id + "-creatediscussion-button-" + ch.roomid, "click", YAHOO.example.container.dialogCreateDiscussion.createDiscussionShow, YAHOO.example.container.dialogCreateDiscussion, true);
				}
			}else{
				var emptyRoster = '<div id="${args.htmlid}-chatslist" class="body scrollableList"><div class="detail-list-item first-item last-item">' + this.msg("label.noChats") + ' !</div></div>';
				con.innerHTML = emptyRoster;
			}
		},
		
		onExistingChatLinkClick: function (e)
        {	
			this.currentview = CV_CHA;
			this.roomid = this.getRoomidFromExistingChatLink(e.currentTarget.id);
			this.openedRoom = this.roomid;
            this.showChatRoom();
        },
		
		onCloseChatLinkClick: function (e){
			this.currentview = CV_CHS;
			var closerid = this.getRoomIdFromCloseChatLink(e.currentTarget.id);
			var id = this.getRoomsObjectId(closerid, this.rooms);
			this.rooms.splice(id,1);
			var postDataCR = '{"rid": ' + this.chatrid + ',"sid":"' + this.chatsid + '","body":[{"type":"presence","from":"' + this.jid + '", "to":"' + closerid + '","close":"true"}]}'; 
			Connect.asyncRequest('POST', this.sUrl, {success: this.handleSuccess, failure: this.handleFailure, argument: this}, postDataCR);
			this.chatrid++;
			this.showChatsList(this.widgets.range2._button.textContent);
		},
		
		onChatLinkClick: function (e)
        {	
			this.currentview = CV_CHA;
            this.openChatRoom(e);
        },
        
		getRooms: function(s,r){
			var postDataGR = '{"rid": ' + r + ',"sid":"' + s + '","body":[{"type":"iq","from":"' + this.jid + '", "to":"' + this.chatsiteorg + '"}]}'; 
			Connect.asyncRequest('POST', this.sUrl, {success: this.handleSuccess, failure: this.handleFailure, argument: this}, postDataGR);
			this.chatrid++;
		},
		
		getRoster: function(filter){
		    var myurl = "";
			if(filter == this.msg("filter.contacts.site")){
				myurl = Alfresco.constants.PROXY_URI + "api/sites/" + this.options.siteId + "/memberships?";
				this.temp_filter = "site";
			}
			if(filter == this.msg("filter.contacts.all")){
				myurl = Alfresco.constants.PROXY_URI + "api/people";
				this.temp_filter= "all";
			}
			Alfresco.util.Ajax.request(
			{
				url: myurl,
				successCallback:
				{
					fn: this.getMembersSuccess,
					scope: this
				}
			});
		},
		
		getPeopleSearchATC: function(query){
		    Alfresco.util.Ajax.request(
			{
				url: Alfresco.constants.PROXY_URI + "api/people?filter=" + encodeURIComponent(query),
				successCallback:
				{
					fn: this.getPeopleSuccess,
					scope: this
				}
			});
		},
		
		getPeopleSuccess: function(resp){
			var ps = [];
			ps = resp.json.people;
			if(ps.length > 0){
				var remove = ["guest"];
				var id = this.getRoomsObjectId(this.roomid, this.rooms);
				for(var k=0;k<this.rooms[id].occupants.length;k++){
					var uname = this.getUsernameFromJid(this.rooms[id].occupants[k]);
					remove.push(uname);
				}
				for(var h=0;h<remove.length;h++){
					var index = ps.length + 1;
					for (var i=0; i< ps.length; i++)
					{
						var p = ps[i];
						if(p.userName == remove[h])
						{
							index = i;	
							break;
						}
					}
					ps.splice(index,1);
				}
				ps.sort(this.sortByName);
			}
			this.searchATC = ps;
			
			var con = Dom.get(this.id + '-addtochat-container');
			con.innerHTML = "";
			if(this.searchATC.length > 0){
				var list = document.createElement('table');
				list.setAttribute('id',this.id + '-addtochat-list');
				con.appendChild(list);
				for (var i=0; i<this.searchATC.length; i++){
					var member = this.searchATC[i];
					var newdiv = document.createElement('tr');
					list.appendChild(newdiv);
					newdiv.innerHTML = '<td><input type="checkbox" id="' + this.id + '-addtochat-checkbox-' + member.userName + '" value="' + member.userName + '"><td>';
					var ava = member.avatar;
					var str = "";
					if(ava !== undefined){
						str = '<img height="32px" width="32px" src="' + Alfresco.constants.URL_CONTEXT + 'proxy/alfresco/' + member.avatar + '?c=force" alt="Avatar" />';
					}else{
						str = '<img height="32px" width="32px" src="' + Alfresco.constants.URL_CONTEXT + 'components/images/no-user-photo-64.png" alt="Avatar" />';
					}
					newdiv.innerHTML += '<td><div class="avatar">' + str + '</div></td>';
					var fname = member.firstName + ' ' + member.lastName;
					newdiv.innerHTML += '<td><div class="person"><span style="font-weight: bold;">' + fname + '</span> (' + member.userName + ')</div></td>';
				}
			}
		},
		
		getMembersSuccess: function(resp){
			var selected = this.temp_filter;
			this.temp_filter = "";
			
			if(selected == "site"){
				var ms = [];
				ms = resp.json;
				for (var i=0; i< ms.length; i++)
				{
					var member = ms[i];
					if(member.authority.userName == Alfresco.constants.USERNAME)
					{
						index = i;	
						break;
					}
				}
				ms.splice(index,1);
				ms.sort(this.sortByName);
				this.roster = ms;
			}
			
			if(selected == "all"){
				var ps = [];
				ps = resp.json.people;
				var remove = ["guest", Alfresco.constants.USERNAME];
				for(var h=0;h<remove.length;h++){
					var index = ps.length + 1;
					for (var i=0; i< ps.length; i++)
					{
						var p = ps[i];
						if(p.userName == remove[h])
						{
							index = i;	
							break;
						}
					}
					ps.splice(index,1);
				}
				ps.sort(this.sortByNamePeople);
				this.roster = ps;
			}
			
			var con = Dom.get(this.id + '-container');
			con.innerHTML = "";
			if(this.roster.length > 0){
				var divusers = document.createElement('div');
				divusers.setAttribute('id',this.id + '-chatuserslist');
				divusers.setAttribute('class','body scrollableList');
				con.appendChild(divusers);
				for (var i=0; i<this.roster.length; i++){
					var member = this.roster[i];
					var ava;
					var fname;
					var username;
					var role;
					var userstatus;
					if(selected == "site"){
						ava = member.authority.avatar;
						fname = member.authority.firstName + ' ' + member.authority.lastName;
						username = member.authority.userName;
						role = this.msg("role." + member.role);
						userstatus = member.authority.userStatus;
					}
					if(selected == "all"){
						ava = member.avatar;
						fname = member.firstName + ' ' + member.lastName;
						username = member.userName;
						role = "&nbsp;";
						userstatus = undefined;
					}
					
					var newdiv = document.createElement('div');
					if(i == 0){
						if(this.roster.length == 1) newdiv.setAttribute('class', 'detail-list-item first-item last-item');
						else newdiv.setAttribute('class', 'detail-list-item first-item');
					}
					if(i == this.roster.length - 1 && i > 0) newdiv.setAttribute('class', 'detail-list-item last-item');
					if(i !== this.roster.length - 1 && i > 0) newdiv.setAttribute('class', 'detail-list-item');
					divusers.appendChild(newdiv);
					
					var divavatar = document.createElement('div');
					divavatar.setAttribute('class','avatar');
					if(ava !== undefined){
						divavatar.innerHTML = '<img src="' + Alfresco.constants.URL_CONTEXT + 'proxy/alfresco/' + ava + '?c=force" alt="Avatar" />';
					}else{
						divavatar.innerHTML = '<img src="' + Alfresco.constants.URL_CONTEXT + 'components/images/no-user-photo-64.png" alt="Avatar" />';
					}
					newdiv.appendChild(divavatar);
										
					var divperson = document.createElement('div');
					divperson.setAttribute('class','person');
					var str = '<h4><a id="' + this.id + '-openchatlink-' + username + '" href="javascript: void(0)" class="theme-color-1 openchatlink">' + fname + '</a></h4>';
					divperson.innerHTML += str;
					str = '<div>' + role + '</div>';
					divperson.innerHTML += str;
					if(userstatus != undefined){
						str = '<div class="user-status">' + userstatus + '</div>';
					}else{
						str = '<div class="user-status"></div>';
					}
					divperson.innerHTML += str;
					newdiv.appendChild(divperson);
					Event.on(this.id + "-openchatlink-" + username, "click", this.onChatLinkClick, null, this);
				}
			}else{
				var emptyRoster = '<div id="${args.htmlid}-chatuserlist" class="body scrollableList"><div class="detail-list-item first-item last-item">' + this.msg("label.noMembers") + ' !</div></div>';
				con.innerHTML = emptyRoster;
			}
		},
		
		sortByName: function(membership1, membership2){
			var name1 = membership1.person ? membership1.person.firstName + membership1.person.lastName : "";
			var name2 = membership2.person ? membership2.person.firstName + membership2.person.lastName : "";
			return (name1 > name2) ? 1 : (name1 < name2) ? -1 : 0;
		},
		
		sortByNamePeople: function(p1, p2){
			var name1 = p1 ? p1.firstName + p1.lastName : "";
			var name2 = p2 ? p2.firstName + p2.lastName : "";
			return (name1 > name2) ? 1 : (name1 < name2) ? -1 : 0;
		},
		
		showChatRoom: function(){
			var con = Dom.get(this.id + '-container');
			con.innerHTML = "";
			var divchat = document.createElement('div');
			divchat.setAttribute('id', this.id + '-chat');
			var inhtml = '<div id="' + this.id + '-chatmessageslist" class="body scrollableList"></div>';
			inhtml += '<form action="" method="post" onsubmit="return false;"><input id="' + this.id + '-inputnewmessage" style="width:99%" type="text" value=""></form>';
			divchat.innerHTML = inhtml;
			con.appendChild(divchat);
			document.getElementById(this.id + '-inputnewmessage').focus(); 
			Event.on(this.id + "-inputnewmessage", "keypress", this.enterNewMessage, null, this);
			var divchatmembers = document.createElement('div');
			divchatmembers.setAttribute('id', this.id + '-chatlistwith');
			divchatmembers.innerHTML = '<b>' + this.msg("chatlist.with") + ':</b> ';
			con.appendChild(divchatmembers);
			if(this.roomid !== null){
				if(this.openedRoom == this.roomid) this.getRoomHistory(this.roomid);
				var id = this.getRoomsObjectId(this.roomid, this.rooms);
				this.rooms[id].newmsg = 0;
				var divsubject = Dom.get(this.id + '-title');
				divsubject.innerHTML = ' [' + this.rooms[id].subject +']';
				this.setLinksInOneChat(this);
				var t = false;
				for(var j=0;j<this.rooms[id].occupants.length;j++){
					var occ = this.rooms[id].occupants[j];
					if(this.jid !== occ){
						if(t !== false) divchatmembers.innerHTML += ', ' + this.getFullnameFromJid(occ);
						else {
							divchatmembers.innerHTML += this.getFullnameFromJid(occ);
							t = true;
						}
					}
				}
			}
			else{
				var divsubject = Dom.get(this.id + '-title');
				divsubject.innerHTML = ' [' + this.startingSubject +']';
				var divtoolbar = Dom.get(this.id + '-toolbarlinks');
				divtoolbar.innerHTML = "";
				divchatmembers.innerHTML += this.startingSubject;
			}
		},
		
		setLinksInOneChat: function(mythis){
			var divtoolbar = Dom.get(mythis.id + '-toolbarlinks');
			divtoolbar.innerHTML = '<a id="' + mythis.id + '-addtochat-button" href="javascript: void(0)" class="theme-color-1">' + mythis.msg("link.addtochat") + '</a>';
			divtoolbar.innerHTML += '<span>  |  </span>';
			divtoolbar.innerHTML += '<a id="' + mythis.id + '-publish-button" href="javascript: void(0)" class="theme-color-1">' + mythis.msg("link.publish") + '</a>';
			divtoolbar.innerHTML += '<span>  |  </span>';
			divtoolbar.innerHTML += '<a id="' + mythis.id + '-creatediscussion-button" href="javascript: void(0)" class="theme-color-1">' + mythis.msg("link.creatediscussion") + '</a>';
			divtoolbar.innerHTML += '<span>  |  </span>';
			divtoolbar.innerHTML += '<a id="' + mythis.id + '-changesubject-button" href="javascript: void(0)" class="theme-color-1">' + mythis.msg("link.changesubject") + '</a>';
			
			YAHOO.util.Event.addListener(this.id + "-addtochat-button", "click", YAHOO.example.container.dialogAddToChat.addToChatShow, YAHOO.example.container.dialogAddToChat, true);
			Event.on(this.id + "-dialogAddToChat-search-button", "click", this.onAddToChatDialogSearchClick, null, this);
			YAHOO.util.Event.addListener(this.id + "-publish-button", "click", YAHOO.example.container.dialogPublish.publishShow, YAHOO.example.container.dialogPublish, true);
			YAHOO.util.Event.addListener(this.id + "-creatediscussion-button", "click", YAHOO.example.container.dialogCreateDiscussion.createDiscussionShow, YAHOO.example.container.dialogCreateDiscussion, true);
			YAHOO.util.Event.addListener(mythis.id + "-changesubject-button", "click", YAHOO.example.container.dialogChangeSub.changeSubShow, YAHOO.example.container.dialogChangeSub, true);
		},
		
		openChatRoom: function(e){
			var uname = this.getUsernameFromChatLink(e.currentTarget.id);
			this.otherid = uname + '@' + this.chatsiteorg;
			this.startingSubject = this.getFullnameFromJid(this.otherid);
			this.roomid = null;
			this.openedRoom = "STR";
			this.showChatRoom();
		},
		
		getRoomHistory: function(id){
			var postDataRH = '{"rid": ' + this.chatrid + ',"sid":"' + this.chatsid + '","body":[{"type":"presence","from":"' + this.jid + '", "to":"' + id + '"}]}'; 
			Connect.asyncRequest('POST', this.sUrl, {success: this.handleSuccess, failure: this.handleFailure, argument: this}, postDataRH);
			this.chatrid++;
		},
		
		createDiscussionTopic: function(mythis, obj){
			Alfresco.util.Ajax.jsonRequest(
			{
				url: Alfresco.constants.PROXY_URI + "api/forum/site/" + mythis.options.siteId + "/discussions/posts",
				method: "POST",
				dataObj: obj,
				successMessage: mythis.msg("alert.creatediscussion.success"),
				failureMessage: mythis.msg("alert.creatediscussion.failure")
			});
		},
		
		createDocumentFromHistory: function(id, mythis, roomhistory){
			var postDataRHJ = '{"rid": ' + mythis.chatrid + ',"sid":"' + mythis.chatsid + '","body":[{"type":"iq","from":"' + mythis.jid + '", "to":"' + id + '"}]}';
			mythis.temp_history = roomhistory;
			Connect.asyncRequest('POST', mythis.sUrl, {success: mythis.handleSuccessHistory, failure: mythis.handleFailure, argument: mythis}, postDataRHJ);
			mythis.chatrid++;
		},
		
		handleSuccessHistory: function(o){
			if(o.responseText !== undefined){
				YAHOO.log("handle success history");
				var jsonr = YAHOO.lang.JSON.parse(o.responseText);
				var roomhistory = o.argument.temp_history;
				roomhistory.history = jsonr.body[0].history;
				o.argument.temp_history = null;
				Alfresco.util.Ajax.jsonRequest(
				{
					url: Alfresco.constants.PROXY_URI + "api/chat/site/" + o.argument.options.siteId + "/documentLibrary/history",
					method: "POST",
					dataObj: roomhistory,
					successMessage: o.argument.msg("alert.publish.success"),
					failureMessage: o.argument.msg("alert.publish.failure")
				});
			}
		},
		
		enterNewMessage: function (e)
        {
			if(e.keyCode == '13'){
				var text = Dom.get(this.id + '-inputnewmessage');
				var value = text.value;
				if(this.roomid == null){
					this.sendMessage(text, this.otherid);
					this.startingSubject = "";
					var divsubject = Dom.get(this.id + '-title');
					divsubject.innerHTML = ' [' + value +']';
				}
				else{
					this.sendMessage(text, this.roomid);
				}
			}
        },
		
		makeConnection: function ()
		{
			YAHOO.log("Make connection handler");
			var postDataSC = '{"rid": 0,"from":"' + this.jid + '"}'; 
			Connect.asyncRequest('POST', this.sUrl, {success: this.handleSuccessStart, failure: this.handleFailure, argument: this}, postDataSC);
			this.chatrid++; 
		},
		
		handleSuccessStart: function(o){
			if(o.responseText !== undefined){
				YAHOO.log("handle success start");
				var jsonr = YAHOO.lang.JSON.parse(o.responseText);
				o.argument.chatsid = jsonr.sid;
				var postDataGR = '{"rid": ' + o.argument.chatrid + ',"sid":"' + o.argument.chatsid + '","body":[{"type":"iq","from":"' + o.argument.jid + '", "to":"' + o.argument.chatsiteorg + '", "flush":"true"}]}'; 
				Connect.asyncRequest('POST', o.argument.sUrl, {success: o.argument.handleSuccessLoop, failure: o.argument.handleFailure, argument: o.argument}, postDataGR);
				o.argument.chatrid++;
			}
		},
		
		handleSuccess: function (o)
		{
			if(o.responseText !== undefined){
				YAHOO.log("handle success");
				var jsonr = YAHOO.lang.JSON.parse(o.responseText);
				o.argument.processResponse(jsonr, o.argument);
			}
		},
		
		handleSuccessLoop: function (o)
		{
			if(o.responseText !== undefined){
				YAHOO.log("handle success loop");
				var jsonr = YAHOO.lang.JSON.parse(o.responseText);
				o.argument.processResponse(jsonr, o.argument);
				Connect.asyncRequest('POST', o.argument.sUrl, {success: o.argument.handleSuccessLoop, failure: o.argument.handleFailure, argument: o.argument}, o.argument.getEmptyPostData(o.argument.chatsid, o.argument.chatrid, o.argument.openedRoom));
				o.argument.chatrid++;
			}
		},
		
		sendMessage: function (eid, to){
			var ta_nm = eid; 
			if(this.chatsid !== null){
				if(!this.isBlank(ta_nm.value)){
					var postDataNM = '{"sid":"'+ this.chatsid + '", "rid": ' + this.chatrid + ', "body":[{"type":"message","from":"' + this.jid +'","to":"'+ to +'","body":"' + ta_nm.value + '"}]}';
					Connect.asyncRequest('POST', this.sUrl, {success: this.handleSuccess, failure: this.handleFailure,  argument: this}, postDataNM);
					ta_nm.value = "";
					this.chatrid++;
				}
			}
		},
		
		sendNewSubject: function(text, to, mythis){
			var postDataNSUB = '{"sid":"'+ mythis.chatsid + '", "rid": ' + mythis.chatrid + ', "body":[{"type":"message","from":"' + mythis.jid +'","to":"'+ to +'","subject":"' + text + '"}]}';
			Connect.asyncRequest('POST', mythis.sUrl, {success: mythis.handleSuccess, failure: mythis.handleFailure,  argument: mythis}, postDataNSUB);
			mythis.chatrid++;
		},
		
		processResponse: function (obj, mythis)
		{
			if(obj.body !== undefined && obj.body !== "[]"){
				var rbody = obj.body;
				for (var i = 0; i < rbody.length; i++) {
					var p = rbody[i];
					var OR = mythis.openedRoom;
					var CV = mythis.currentview;
					if(p.type === "message"){
						if(p.subject !== undefined){
							if(OR !== "STR" && OR !== "NOR" && OR == mythis.roomid){
								if(p.to == OR){
									var id = mythis.getRoomsObjectId(mythis.roomid, mythis.rooms);
									mythis.rooms[id].subject = p.subject;
									var divsubject = Dom.get(mythis.id + '-title');
									divsubject.innerHTML = ' [' + p.subject +']';
									var divchat = Dom.get(mythis.id + '-chatmessageslist');
									var color = 'red';
									if(mythis.jid === p.from) color = 'blue';
									var fullname = mythis.getFullnameFromJid(p.from);
									var isDiff = mythis.isFromDifferentDay(p.stamp);
									var time = mythis.getTimeFromTS(p.stamp);
									if(isDiff == true){
										time = mythis.getFullDate(p.stamp);
									}
									divchat.innerHTML += '<li style="color:' + color + '">' + fullname + ' ' + mythis.msg("chat.changesubject") + ' "' + p.subject + '" (' + time + ')</li>';
								}else{
									var id = mythis.getRoomsObjectId(p.to, mythis.rooms);
									mythis.rooms[id].subject = p.subject;
									if(p.history == undefined){
										mythis.rooms[id].newmsg += 1;
									}
								}
							}
							
							if(OR == "NOR" && CV !== CV_CHA){
								var id = mythis.getRoomsObjectId(p.to, mythis.rooms);
								mythis.rooms[id].subject = p.subject;
								mythis.rooms[id].newmsg += 1;
								if(CV == CV_CHS){
									var divsubject = Dom.get(mythis.id + '-openchatlinkFL-' + mythis.rooms[id].roomid);
									if(divsubject != null){
										divsubject.innerHTML = p.subject;
										var divnewmsg = Dom.get(mythis.id + '-newmsg-' + mythis.rooms[id].roomid);
										divnewmsg.innerHTML = mythis.msg("chat.new") + ': ';
										if(mythis.rooms[id].newmsg > 0){ divnewmsg.innerHTML += '<span style="color: red;">' + mythis.rooms[id].newmsg + '</span>';}
										else {divnewmsg.innerHTML += mythis.rooms[id].newmsg;}
									}else{
										mythis.showChatsList(mythis.widgets.range2._button.textContent);
									}
								}
							}
						}
						else{
							if(OR !== "STR" && OR !== "NOR" && OR == mythis.roomid){
								if(p.to == OR){
									var divchat = Dom.get(mythis.id + '-chatmessageslist');
									var color = 'red';
									if(mythis.jid === p.from) color = 'blue';
									var fullname = mythis.getFullnameFromJid(p.from); 
									var isDiff = mythis.isFromDifferentDay(p.stamp);
									var time = mythis.getTimeFromTS(p.stamp);;
									if(isDiff == true){
										time = mythis.getFullDate(p.stamp);
									}
									divchat.innerHTML += '<li style="color:' + color + '">' + fullname + ' ' + mythis.msg("chat.sendmessage") + ' (' + time + '):</li>';
									divchat.innerHTML += "<li>" + p.body + "</li>";
								}else{
									var id = mythis.getRoomsObjectId(p.to, mythis.rooms);
									if(p.history == undefined){
										mythis.rooms[id].newmsg += 1;
									}
								}
							}
							
							if(OR == "NOR" && CV !== CV_CHA){
								var id = mythis.getRoomsObjectId(p.to, mythis.rooms);
								if(CV == CV_CHS){
									if(p.history == undefined){
										mythis.rooms[id].newmsg += 1;
										var divrp = Dom.get(mythis.id + '-newmsg-' + mythis.rooms[id].roomid);
										if(divrp != null){
											divrp.innerHTML = mythis.msg("chat.new") + ': ';
											if(mythis.rooms[id].newmsg > 0){ divrp.innerHTML += '<span style="color: red;">' + mythis.rooms[id].newmsg + '</span>';}
											else {divrp.innerHTML += mythis.rooms[id].newmsg;}
										}else{
											mythis.showChatsList(mythis.widgets.range2._button.textContent);
										}
									}
								}else{
									if(p.history == undefined){
										mythis.rooms[id].newmsg += 1;
									}
								}
							}
						}
					}
					if(p.type === "presence"){
						var id = mythis.getRoomsObjectId(p.to, mythis.rooms);
						if(id !== null){
							var found = null;
							for(var h=0;h<mythis.rooms[id].occupants.length;h++){
								if(mythis.rooms[id].occupants[h] == p.from){
									found = h;
									break;
								}
							}
							if(found == null){
								mythis.rooms[id].occupants.push(p.from);
								if(OR == "NOR" && CV == CV_CHS){
									var divperson = Dom.get(mythis.id + '-chatlistwith-' + mythis.rooms[id].roomid);
									if(divperson != null){
										divperson.innerHTML += '<li>' + mythis.getFullnameFromJid(p.from); + '</li>';
									}else{
										mythis.showChatsList(mythis.widgets.range2._button.textContent);
									}
								}
								if(OR !== "STR" && OR !== "NOR" && OR == mythis.roomid){
									var divmembers = Dom.get(mythis.id + '-chatlistwith');
									if(divmembers != null && mythis.jid !== p.from){
										var len = mythis.rooms[id].occupants.length;
										if(len < 3){
											divmembers.innerHTML += mythis.getFullnameFromJid(p.from);
										}
										else {
											divmembers.innerHTML += ', ' + mythis.getFullnameFromJid(p.from);
										}
									}
								}
							}else{
								if(p.close !== undefined){
									mythis.rooms[id].occupants.splice(found,1);
									if(OR == "NOR" && CV == CV_CHS){
										mythis.showChatsList(mythis.widgets.range2._button.textContent);
									}
								}
							}
						}else{
							var room = {
								roomid: p.to,
								subject: p.subject,
								occupants: [p.from],
								newmsg: 0
							};
							mythis.rooms.push(room);
							if(OR == "NOR" && CV == CV_CHS){
								mythis.showChatsList(mythis.widgets.range2._button.textContent);
							}
							if(OR == "STR" && CV == CV_CHA && p.from == mythis.jid){
								mythis.roomid = p.to;
								mythis.openedRoom = p.to;
								mythis.otherid = null;
								mythis.setLinksInOneChat(mythis);
								var divmembers = Dom.get(mythis.id + '-chatlistwith');
								if(divmembers != null){
									divmembers.innerHTML = '<b>' + this.msg("chatlist.with") + ':</b> ';
								}
							}
						}
					}
					if(p.type === "result"){
						mythis.rooms = p.items;
					}
				}
			}
		},
		
		handleFailure: function (o)
		{
			//alert('The failure handler was called !' + 'Kod: ' + o.status + ' Text: ' + o.statusText);
		},
		
		isBlank: function (str)
		{
			return (!str || /^\s*$/.test(str));
		},
		
		getRoomsObjectId: function (id, rms){
			var room = null;
			for(var i=0;i<rms.length;i++){
				if(rms[i].roomid == id){
					room = i;
					break;
				}
			}
			return room;
		},

		getEmptyPostData: function (mysid, myrid, myOpenedRoom)
		{
			var myPostData = '{"sid":"' + mysid + '", "rid": ' + myrid + ', '; 
			if(myOpenedRoom !== "NOR" && myOpenedRoom !== "STR") myPostData += '"openedRoom": ' + '"' + myOpenedRoom + '", ';
			myPostData += '"body":"[]"}';
			return myPostData;
		},
		
		getTimeFromTS: function (ts)
		{
			var tsplit = ts.split("T");
			var tsplit2 = tsplit[1].split(".");
			return tsplit2[0];
		},
		
		isFromDifferentDay: function (ts)
		{
			var split = ts.split("-");
			var year = split[0];
			var month = split[1];
			var split2 = split[2].split("T");
			var day = split2[0];
			var date = new Date();
			var dn = date.getDate();
			var dm = date.getMonth();
			var dy = date.getFullYear();
			if(day !== dn.toString()) {return true;}
			else{
				var m = "";
				if(dm > 8) {m = (dm + 1).toString();}
				else {m = "0" + (dm + 1).toString();}
				if(month !== m) {return true;}
				else{
					if(year !== dy.toString()) {return true}
					else {return false;}
				}
			}
		},
		
		getFullDate: function (ts)
		{
			var split = ts.split("-");
			var year = split[0];
			var month = split[1];
			var split2 = split[2].split("T");
			var day = split2[0];
			var tsplit = ts.split("T");
			var tsplit2 = tsplit[1].split(".");
			var str = day + '-' + month + '-' + year + ' ' + tsplit2[0];
			return str;
		},
		
		getFullnameFromJid: function (myjid){
			var mySplitResult = myjid.split("@");
			var uname = mySplitResult[0];
			for(var k=0;k<this.users.length;k++){
				var u = this.users[k];
				if(u.username == uname){
					return u.fullname;
				}
			}
			this.getUsers();
			return uname;
		},
		
		getUsernameFromJid: function (myjid){
			var mySplitResult = myjid.split("@");
			return mySplitResult[0];
		},
		
		getRoomidFromExistingChatLink: function(eid){
			var mySplitResult = eid.split("-openchatlinkFL-");
			return mySplitResult[1];
		},
		
		getRoomIdFromCloseChatLink: function(eid){
			var mySplitResult = eid.split("-closechatlinkFL-");
			return mySplitResult[1];
		},
		
		getUsernameFromChatLink: function (eid){
			var mySplitResult = eid.split("-openchatlink-");
			return mySplitResult[1];
		},
		
		getRoomidFromPublishLink: function (eid){
			var mySplitResult = eid.split("-publish-button-");
			return mySplitResult[1];
		},
		
		getRoomidFromCreateDiscussionLink: function (eid){
			var mySplitResult = eid.split("-creatediscussion-button-");
			return mySplitResult[1];
		},
    });
})();